package com.ruoyi.ss.payBill.service;

import java.time.LocalDateTime;
import java.util.List;
import com.ruoyi.ss.payBill.domain.PayBill;
import com.ruoyi.ss.payBill.domain.PayBillVO;
import com.ruoyi.ss.payBill.domain.PayBillQuery;
import com.ruoyi.ss.payBill.domain.enums.PayBillBstType;
import com.ruoyi.ss.timeBill.domain.TimeBillVO;

/**
 * 支付订单Service接口
 *
 * @author ruoyi
 * @date 2024-07-24
 */
public interface PayBillService
{
    /**
     * 查询支付订单
     *
     * @param payId 支付订单主键
     * @return 支付订单
     */
    public PayBillVO selectPayBillByPayId(Long payId);

    /**
     * 查询支付订单列表
     *
     * @param payBill 支付订单
     * @return 支付订单集合
     */
    public List<PayBillVO> selectPayBillList(PayBillQuery payBill);

    /**
     * 新增支付订单
     *
     * @param payBill 支付订单
     * @return 结果
     */
    public int insertPayBill(PayBill payBill);

    /**
     * 修改支付订单
     *
     * @param payBill 支付订单
     * @return 结果
     */
    public int updatePayBill(PayBill payBill);

    /**
     * 批量删除支付订单
     *
     * @param payIds 需要删除的支付订单主键集合
     * @return 结果
     */
    public int deletePayBillByPayIds(Long[] payIds);

    /**
     * 删除支付订单信息
     *
     * @param payId 支付订单主键
     * @return 结果
     */
    public int deletePayBillByPayId(Long payId);

    /**
     * 创建支付订单
     */
    PayBillVO createBill(TimeBillVO bill, Long channelId);

    /**
     * 根据业务ID关闭订单
     *
     * @param bstId   业务ID
     * @param bstType 业务类型
     */
    boolean closeByBstId(Long bstId, PayBillBstType bstType);

    /**
     * 根据业务ID查询
     */
    List<PayBillVO> selectByBstId(Long bstId, String bstType);

    /**
     * 根据支付订单编号查询
     */
    PayBillVO selectByPayNo(String payNo);

    /**
     * 根据条件更新
     */
    int updateByQuery(PayBill data, PayBillQuery query);

    /**
     * 处理支付成功时间
     * @param payNo 支付订单编号
     * @param payTime 支付成功的时间
     */
    void handleSuccess(String payNo, LocalDateTime payTime);
}
