package com.ruoyi.ss.transactionBill.service.impl;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.enums.WithdrawServiceType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.account.domain.AccountVO;
import com.ruoyi.ss.account.service.AccountService;
import com.ruoyi.ss.channel.domain.ChannelVO;
import com.ruoyi.ss.channel.service.ChannelService;
import com.ruoyi.ss.dashboard.BillCountVo;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.receiveBill.service.ReceiveBillService;
import com.ruoyi.ss.record.time.service.IRecordTimeService;
import com.ruoyi.ss.record.time.service.RecordTimeConverter;
import com.ruoyi.ss.refund.domain.Refund;
import com.ruoyi.ss.refund.domain.RefundVO;
import com.ruoyi.ss.refund.service.RefundConverter;
import com.ruoyi.ss.refund.service.RefundService;
import com.ruoyi.ss.store.domain.StoreVo;
import com.ruoyi.ss.store.service.IStoreService;
import com.ruoyi.ss.suit.domain.SuitVO;
import com.ruoyi.ss.suit.service.SuitService;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.vo.TransactionBillVO;
import com.ruoyi.ss.transactionBill.domain.bo.RechargeBO;
import com.ruoyi.ss.transactionBill.domain.vo.UserWithdrawServiceVO;
import com.ruoyi.ss.transactionBill.domain.bo.WithdrawBO;
import com.ruoyi.ss.transactionBill.domain.dto.BillRefundDTO;
import com.ruoyi.ss.transactionBill.domain.dto.WithdrawApprovalDTO;
import com.ruoyi.ss.transactionBill.domain.dto.WithdrawDTO;
import com.ruoyi.ss.transactionBill.domain.enums.*;
import com.ruoyi.ss.transactionBill.mapper.TransactionBillMapper;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.transactionBill.service.TransactionBillValidator;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.common.enums.ServiceType;
import com.ruoyi.ss.user.mapper.SmUserMapper;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.common.pay.wx.service.WxPayService;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.task.bill.BillDelayedManager;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.transferbatch.model.InitiateBatchTransferResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 充值记录Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Service("transactionBillService")
@Slf4j
public class TransactionBillServiceImpl implements TransactionBillService {
    @Autowired
    private TransactionBillMapper transactionBillMapper;

    @Autowired
    private ChannelService channelService;

    @Autowired
    private TransactionBillValidator transactionBillValidator;

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private WxPayService wxPayService;

    @Autowired
    private ISmUserService userService;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    private BillDelayedManager billDelayedManager;

    @Autowired
    private SmUserMapper smUserMapper;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private AccountService smAccountService;

    @Autowired
    private IStoreService storeService;

    @Autowired
    private SuitService suitService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Value("${sm.withdraw.password.enable}")
    private Boolean smWithdrawPasswordEnable;   // 是否开启提现密码

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IRecordTimeService recordTimeService;

    @Autowired
    private RecordTimeConverter recordTimeConverter;

    @Autowired
    private RefundService refundService;

    @Autowired
    private RefundConverter refundConverter;

    @Autowired
    private ReceiveBillService receiveBillService;

    /**
     * 查询充值记录
     *
     * @param billId 充值记录主键
     * @return 充值记录
     */
    @Override
    public TransactionBillVO selectSmTransactionBillByBillId(Long billId)
    {
        return transactionBillMapper.selectSmTransactionBillByBillId(billId);
    }

    /**
     * 查询充值记录列表
     *
     * @param smTransactionBillQuery 充值记录查询条件
     * @return 充值记录
     */
    @Override
    public List<TransactionBillVO> selectSmTransactionBillList(TransactionBillQuery smTransactionBillQuery)
    {
        return transactionBillMapper.selectSmTransactionBillList(smTransactionBillQuery);
    }

    /**
     * 新增充值记录
     *
     * @param transactionBill 充值记录
     * @return 结果
     */
    @Override
    public int insertSmTransactionBill(TransactionBill transactionBill)
    {
        transactionBill.setCreateTime(DateUtils.getNowDate());
        transactionBill.setBillNo(String.valueOf(SnowFlakeUtil.newId()));
        return transactionBillMapper.insertSmTransactionBill(transactionBill);
    }

    /**
     * 修改充值记录
     *
     * @param transactionBill 充值记录
     * @return 结果
     */
    @Override
    public int updateSmTransactionBill(TransactionBill transactionBill)
    {
        return transactionBillMapper.updateSmTransactionBill(transactionBill);
    }

    /**
     * 批量删除充值记录
     *
     * @param billIds 需要删除的充值记录主键
     * @return 结果
     */
    @Override
    public int deleteSmTransactionBillByBillIds(Long[] billIds)
    {
        return transactionBillMapper.deleteSmTransactionBillByBillIds(billIds);
    }

    /**
     * 删除充值记录信息
     *
     * @param billId 充值记录主键
     * @return 结果
     */
    @Override
    public int deleteSmTransactionBillByBillId(Long billId)
    {
        return transactionBillMapper.deleteSmTransactionBillByBillId(billId);
    }

    /**
     * 查询账单统计
     * @param dto
     * @return
     */
    @Override
    public List<BillCountVo> selectCount(TransactionBillQuery dto) {
        return transactionBillMapper.selectCount(dto);
    }

    @Override
    public String addOrder(RechargeBO bo) {
        // 下单
        TransactionBill order = parseToOrder(bo);

        transactionTemplate.execute(status -> {
            // 插入数据库
            int insert = this.insertSmTransactionBill(order);
            ServiceUtil.assertion(insert != 1, "下单失败");
            return insert;
        });

        // 若订单金额为0元，则直接充值成功
        if (order.getMoney().compareTo(BigDecimal.ZERO) == 0) {
            TransactionBillVO bill = this.selectSmTransactionBillByBillNo(order.getBillNo());

            // 更新为支付中
            transactionTemplate.execute(status -> {
                int paying = this.paying(bill.getBillId());
                ServiceUtil.assertion(paying != 1, "更新支付状态失败");
                return paying;
            });

            this.rechargeSuccess(order.getBillNo(), DateUtils.getNowDate());
        }

        // 加入延时队列，超时取消
        billDelayedManager.add(order.getBillNo(), order.getExpireTime().getTime());

        return order.getBillNo();
    }

    // 处理充值服务费
    private void handleRechargeService(TransactionBill order, ChannelVO channel, SmUserVo mch, DeviceVO device) {
        ServiceUtil.assertion(mch == null, "商户不存在，不允许下单");
        ServiceUtil.assertion(device == null, "设备不存在，不允许下单");

        String serviceType = null;  // 服务费类型
        BigDecimal serviceRate = null;   // 服务费

        // 优先级： 设备 > 商户 > 渠道
        if (device.getServiceRate() != null && StringUtils.hasText(device.getServiceType())) {
            serviceType = device.getServiceType();
            serviceRate = device.getServiceRate();
        }
        // 商户
        else if (mch.getServiceRate() != null && StringUtils.hasText(mch.getServiceType())) {
            serviceType = mch.getServiceType();
            serviceRate = mch.getServiceRate();
        }
        // 渠道
        else {
            ServiceUtil.assertion(channel == null, "支付渠道不存在");
            ServiceUtil.assertion(channel.getServiceRate() == null, "支付渠道服务费未配置，请联系管理员");
            ServiceUtil.assertion(StringUtils.isBlank(channel.getServiceType()), "支付渠道服务费收取方式未配置，请联系管理员");
            serviceType = channel.getServiceType();
            serviceRate = channel.getServiceRate();
        }

        // 月费
        if (ServiceType.MONTH.getType().equals(serviceType)) {
            int count = receiveBillService.genBillByMonthAndPay(mch, device, LocalDateTime.now(), serviceRate);
            ServiceUtil.assertion(count == 0, "商户出账失败，请刷新后重试");
            // 服务费 = 0
            order.setServiceCharge(BigDecimal.ZERO);
            // 商户到账金额 = 交易金额
            order.setArrivalAmount(order.getMoney());
        }
        // 设备订单比例
        else if(ServiceType.PERCENT.getType().equals(serviceType)){
            // 服务费
            BigDecimal serviceCharge = serviceRate.multiply(order.getMoney()).divide(new BigDecimal(100), 2, RoundingMode.HALF_UP);
            order.setServiceCharge(serviceCharge);
            // 商户最终到账的金额 = 交易金额 - 服务费
            order.setArrivalAmount(order.getMoney().subtract(serviceCharge));
        } else {
            throw new ServiceException("未知的服务费类型");
        }

    }

    // 转换为订单所需的数据
    private TransactionBill parseToOrder(RechargeBO bo) {
        // 校验
        ServiceUtil.assertion(transactionBillValidator.preAddOrder(bo));
        TransactionBill data = bo.getParams();
        DeviceVO device = bo.getDevice();
        SmUserVo mch = bo.getMch();
        SuitVO suit = bo.getSuit();
        SmUserVo user = bo.getUser();
        StoreVo store = bo.getStore();
        ChannelVO channel = bo.getChannel();

        TransactionBill order = new TransactionBill();
        // 基础信息
        order.setUserId(data.getUserId());
        order.setSuitTime(data.getSuitTime());
        order.setSuitTimeUnit(data.getSuitTimeUnit());
        order.setDeviceNo(data.getDeviceNo());      // 设备编号
        order.setChannelId(data.getChannelId());
        order.setMoney(data.getMoney());
        order.setSuitId(data.getSuitId());
        order.setType(TransactionBillType.RECHARGE.getType());       // 类型：订单
        order.setStatus(TransactionBillStatus.UNPAID.getStatus());   // 状态：未支付
        order.setDeviceRechargeStatus(TransactionBillDeviceRechargeStatus.UN_RECHARGE.getStatus()); // 设备状态：未充值
        // 设备信息
        order.setDeviceId(device.getDeviceId());
        order.setMchId(device.getUserId());    // 到账人ID
        order.setDeviceName(device.getDeviceName());
        order.setDeviceMac(device.getMac());
        // 店铺信息
        if (store != null) {
            order.setStoreId(store.getStoreId());
            order.setStoreName(store.getName());
            order.setStoreAddress(String.format("%s%s%s%s", store.getProvince(), store.getCity(), store.getCounty(), store.getAddress()));
        }
        // 套餐信息
        order.setSuitName(suit.getName());
        // 用户信息
        order.setAccountNo(user.getWxOpenId());
        // 渠道费用计算
        order.setChannelCost(channel.getCostRate().multiply(order.getMoney()).divide(new BigDecimal(100), 2, RoundingMode.HALF_UP));
        // 充值服务费处理
        this.handleRechargeService(order, channel, mch, device);
        // 支付过期时间
        long expireTime = TimeUnit.MILLISECONDS.convert(Constants.BILL_UNPAID_TIMEOUT, Constants.BILL_UNPAID_TIMEUNIT) + System.currentTimeMillis();
        order.setExpireTime(new Date(expireTime));
        return order;
    }

    @Override
    public String addWithdraw(WithdrawBO bo) {
        // 参数校验
        Long userId = bo.getDto().getUserId();
        ServiceUtil.assertion(userId == null, "参数错误：用户id为空");
        // 加锁
        ServiceUtil.assertion(!redisLock.lock(RedisLockKey.ADD_WITHDRAW, userId), "请勿频繁操作提现");
        try {
            WithdrawDTO dto = bo.getDto();
            ChannelVO channel = bo.getChannel();
            SmUserVo user = bo.getUser();
            AccountVO account = bo.getAccount();

            // 处理提现手续费
            ServiceUtil.assertion(channel == null, "提现渠道不存在");
            ServiceUtil.assertion(channel.getWithdrawEnabled() == null || !channel.getWithdrawEnabled(), "提现渠道不可用");
            UserWithdrawServiceVO serviceInfo = this.getUserWithdrawService(user, channel);
            ServiceUtil.assertion(serviceInfo == null, "服务费配置出错，请联系管理员处理");
            ServiceUtil.assertion(StringUtils.isBlank(serviceInfo.getServiceType()), "服务费配置出错，请联系管理员处理");
            ServiceUtil.assertion(serviceInfo.getServiceRate() == null, "服务费配置出错，请联系管理员处理");
            ServiceUtil.assertion(!Objects.equals(serviceInfo.getServiceType(), dto.getServiceType()), "服务费收取方式发生变化，请刷新后重试");
            ServiceUtil.assertion(serviceInfo.getServiceRate().compareTo(dto.getServiceRate()) != 0, "服务费发生变化，请刷新后重试");

            // 计算服务费
            BigDecimal serviceRecharge = this.calcWithdrawServiceCharge(serviceInfo, dto.getMoney());
            ServiceUtil.assertion(BigDecimal.ZERO.compareTo(serviceRecharge) > 0, "服务费不允许小于0");
            ServiceUtil.assertion(serviceRecharge.compareTo(dto.getServiceCharge()) != 0, "服务费计算不一致，请重新申请");
            // 到账金额
            BigDecimal arrivalAmount = dto.getMoney().subtract(serviceRecharge);   // 到账金额
            ServiceUtil.assertion(arrivalAmount.compareTo(dto.getArrivalAmount()) != 0, "到账金额计算不一致，请重新申请");

            // 创建申请订单
            TransactionBill bill = new TransactionBill();
            bill.setUserId(userId);
            bill.setType(TransactionBillType.WITHDRAW.getType());
            bill.setMchId(userId);
            bill.setMoney(dto.getMoney());
            bill.setArrivalAmount(arrivalAmount);
            bill.setServiceCharge(serviceRecharge);
            bill.setStatus(TransactionBillStatus.WITHDRAW_APPROVING.getStatus());   // 审核中
            bill.setAccountNo(account.getAccountNo());
            bill.setChannelId(channel.getChannelId());
            bill.setWithdrawType(WithdrawType.ONLINE.getType());
            bill.setChannelCost(channel.getWithdrawCostRate().multiply(bill.getMoney()).divide(new BigDecimal(100), 2, RoundingMode.HALF_UP));   // 渠道成本

            transactionTemplate.execute(status -> {
                // 减少余额，并判断提现金额是否超额（减少的是交易金额）
                userService.subtractBalance(userId, bill.getMoney(), "提现");

                // 减少余额后的用户信息
                SmUserVo afterUser = smUserMapper.selectSimpleById(userId);
                bill.setAfterBalance(afterUser.getBalance());

                // 提现申请
                int insert = this.insertSmTransactionBill(bill);
                ServiceUtil.assertion(insert != 1, "提现申请失败");

                return insert;
            });

            return bill.getBillNo();
        } finally {
            redisLock.unlock(RedisLockKey.ADD_WITHDRAW, userId);
        }

    }

    private BigDecimal calcWithdrawServiceCharge(UserWithdrawServiceVO serviceInfo, BigDecimal money) {
        BigDecimal serviceRecharge;
        if (WithdrawServiceType.PERCENT.getType().equals(serviceInfo.getServiceType())) {
            serviceRecharge = money.multiply(serviceInfo.getServiceRate()).divide(new BigDecimal(100), 2, RoundingMode.HALF_UP); // 服务费
        } else if (WithdrawServiceType.FIXED.getType().equals(serviceInfo.getServiceType())) {
            serviceRecharge = serviceInfo.getServiceRate();
        } else {
            throw new ServiceException("服务费配置出错，请联系管理员处理");
        }
        return serviceRecharge;
    }

    /**
     * 获取用户提现服务费
     * 用户 > 渠道
     */
    private UserWithdrawServiceVO getUserWithdrawService(SmUserVo user, ChannelVO channel) {
        if (user != null && StringUtils.hasText(user.getWithdrawServiceType()) && user.getWithdrawServiceRate() != null ) {
            return new UserWithdrawServiceVO(user.getWithdrawServiceType(), user.getWithdrawServiceRate());
        } else if (channel != null && StringUtils.hasText(channel.getWithdrawServiceType()) && channel.getWithdrawServiceRate() != null) {
            return new UserWithdrawServiceVO(channel.getWithdrawServiceType(), channel.getWithdrawServiceRate());
        }
        return null;
    }

    /**
     * 提现审核通过
     *
     * @param dto id
     */
    @Override
    public boolean passWithdraw(WithdrawApprovalDTO dto) {
        ServiceUtil.assertion(dto.getBillId() == null, "提现申请ID不允许为空");
        ServiceUtil.assertion(!redisLock.lock(RedisLockKey.APPROVAL_WITHDRAW, dto.getBillId()), "该提现申请正在审核，请刷新后重试");
        try {
            transactionTemplate.execute(status -> {
                int updateCount = this.approvalWithdraw(dto, TransactionBillStatus.WITHDRAW_PASSED);
                ServiceUtil.assertion(updateCount != 1, "提现审核失败，请刷新后重试");
                return updateCount;
            });

            this.payWithdraw(dto.getBillId());

            return true;
        } finally {
            redisLock.unlock(RedisLockKey.APPROVAL_WITHDRAW, dto.toString());
        }
    }

    /**
     * 审核操作
     */
    private int approvalWithdraw(WithdrawApprovalDTO dto, TransactionBillStatus status) {
        TransactionBill data = new TransactionBill();
        data.setStatus(status.getStatus());
        data.setRemark(dto.getRemark());
        data.setPayPicture(dto.getPayPicture());
        data.setWithdrawType(dto.getWithdrawType());
        TransactionBillQuery query = new TransactionBillQuery();
        query.setBillId(dto.getBillId());
        query.setType(TransactionBillType.WITHDRAW.getType());
        query.setStatus(TransactionBillStatus.WITHDRAW_APPROVING.getStatus());
        return this.updateByQuery(data, query);
    }

    @Override
    public boolean rejectWithdraw(WithdrawApprovalDTO dto) {
        ServiceUtil.assertion(dto.getBillId() == null, "提现申请ID不允许为空");
        ServiceUtil.assertion(!redisLock.lock(RedisLockKey.APPROVAL_WITHDRAW, dto.getBillId()), "该提现申请正在审核，请刷新后重试");
        try {
            transactionTemplate.execute(status -> {
                // 审核
                int updateCount = this.approvalWithdraw(dto, TransactionBillStatus.WITHDRAW_REJECTED);
                ServiceUtil.assertion(updateCount != 1, "提现审核失败，请刷新后重试");

                // 返还客户余额
                TransactionBill bill = transactionBillMapper.selectSmTransactionBillByBillId(dto.getBillId());
                ServiceUtil.assertion(bill == null || bill.getUserId() == null, "数据不存在");
                userService.addBalance(bill.getUserId(), bill.getMoney(), "提现驳回");

                return updateCount;
            });

            return true;
        } finally {
            redisLock.unlock(RedisLockKey.APPROVAL_WITHDRAW, dto.toString());
        }
    }

    /**
     * 提现打款
     * @param billId 提现单id
     */
    @Override
    public boolean payWithdraw(Long billId) {
        ServiceUtil.assertion(billId == null, "参数错误：提现打款id不允许为空");
        ServiceUtil.assertion(!redisLock.lock(RedisLockKey.PAY_WITHDRAW, billId), "该提现申请正在打款，请刷新后重试");
        try {
            TransactionBillVO bill = transactionBillMapper.selectSmTransactionBillByBillId(billId);
            ServiceUtil.assertion(bill == null, "单据不存在");

            transactionTemplate.execute(status -> {
                // 更新为打款中
                int updateCount = transactionBillMapper.doPayWithdraw(billId);
                ServiceUtil.assertion(updateCount != 1, "单据状态发生变化，请刷新后重试");

                // 发起打款（线上）
                if (WithdrawType.ONLINE.getType().equals(bill.getWithdrawType())) {
                    // 微信
                    if (TransactionBillPayType.WECHAT.getType().equals(bill.getChannelId())) {
                        InitiateBatchTransferResponse res = wxPayService.payWithdraw(billId);
                        log.debug(String.valueOf(res));
                    } else {
                        throw new ServiceException("其他打款渠道正在开发中");
                    }
                }
                // 线下
                else if(WithdrawType.OFFLINE.getType().equals(bill.getWithdrawType())) {
                    this.payWithdrawByOffline(bill);
                } else {
                    throw new ServiceException("不支持的打款方式");
                }

                return updateCount;
            });

            return true;
        } finally {
            redisLock.unlock(RedisLockKey.PAY_WITHDRAW, billId);
        }
    }

    /**
     * 线下打款
     */
    private void payWithdrawByOffline(TransactionBillVO bill) {
        this.withdrawSuccess(bill.getBillId(), LocalDateTime.now());
    }

    @Override
    public void withdrawSuccess(Long billId, LocalDateTime payTime) {
        transactionTemplate.execute(status -> {
            // 修改状态
            int updateCount = transactionBillMapper.withdrawSuccess(billId, payTime);
            ServiceUtil.assertion(updateCount != 1, "单据状态发生变化，请刷新后重试");
            return updateCount;
        });
    }

    @Override
    @Transactional
    public void withdrawFailed(Long billId) {
        // 修改状态
        int updateCount = transactionBillMapper.withdrawFailed(billId);
        ServiceUtil.assertion(updateCount != 1, "单据状态发生变化，请刷新后重试");
    }

    /**
     * 更新微信批次明细单号
     * @param billId 提现单号
     * @param transferIds 微信批次明细列表
     */
    @Override
    public boolean updateWxTransferDetailIds(Long billId, List<String> transferIds) {
        ServiceUtil.assertion(billId == null || CollectionUtils.isEmpty(transferIds), "参数错误：提现单号或微信批次明细列表为空");
        TransactionBill bill = new TransactionBill();
        bill.setBillId(billId);
        bill.setTransferIds(transferIds);
        return transactionBillMapper.updateSmTransactionBill(bill) == 1;
    }

    @Override
    public List<BillCountVo> selectLandlordCount(TransactionBillQuery dto) {
        List<BillCountVo> list = this.selectCount(dto);

        // 若startDate与endDate不为空，则将其中间数据补全，不让其为空
        if (dto.getStartDate() != null && dto.getEndDate() != null && Objects.equals(dto.getGroupBy(), TransactionBillGroupBy.create_date.name())) {
            List<BillCountVo> newList = new ArrayList<>();
            Date i = dto.getStartDate();
            while (i.compareTo(dto.getEndDate()) <= 0) {
                Date finalI = i;
                BillCountVo count = list.stream().filter(item -> DateUtils.getYYYY_MM_DD(item.getCreateDate()).equals(DateUtils.getYYYY_MM_DD(finalI))).findFirst().orElse(null);
                if (count == null) {
                    count = new BillCountVo();
                    count.setCreateDate(finalI);
                    count.setServiceCharge(BigDecimal.ZERO);
                    count.setWithdraw(BigDecimal.ZERO);
                    count.setRecharge(BigDecimal.ZERO);
                    count.setCreateYear(DateUtils.getYear(finalI));
                    count.setCreateMonth(DateUtils.getMonth(finalI));
                    count.setCreateDay(DateUtils.getDay(finalI));
                    count.setCreateYearMonth(DateUtils.getYYYY_MM(finalI));
                }
                newList.add(count);
                i = DateUtils.addDays(i,1);
            }

            // 列表排序
            newList.sort(Comparator.comparing(BillCountVo::getCreateDate));
            list = newList;
        }

        return list;
    }

    /**
     * 刷新支付结果
     *
     * @param billId
     * @return
     */
    @Transactional
    public int refreshPayResult(Long billId) {
        return this.refreshPayResult(this.selectSmTransactionBillByBillId(billId));
    }

    @Transactional
    public int refreshPayResult(TransactionBillVO bill) {
        return this.refreshPayResult(Collections.singletonList(bill));
    }

    @Override
    public int paying(Long billId) {
        if (billId == null) {
            return 0;
        }
        return transactionBillMapper.paying(billId);
    }

    /**
     * 设备充值
     *
     * @param billId
     */
    @Override
    public boolean rechargeDevice(Long billId) {
        ServiceUtil.assertion(billId == null, "参数错误,billId不允许为空");
        ServiceUtil.assertion(!redisLock.lock(RedisLockKey.RECHARGE_DEVICE, billId), "当前设备充值请求过于频繁，请等待");
        try {
            TransactionBillVO bill = transactionBillMapper.selectSmTransactionBillByBillId(billId);
            ServiceUtil.assertion(bill == null || !TransactionBillType.RECHARGE.getType().equals(bill.getType()), "不存在的充值订单");
            ServiceUtil.assertion(!TransactionBillStatus.SUCCESS.getStatus().equals(bill.getStatus()), "订单未支付");
            ServiceUtil.assertion(TransactionBillDeviceRechargeStatus.SUCCESS.getStatus().equals(bill.getDeviceRechargeStatus()), "设备已充值成功，不允许再次充值");
            ServiceUtil.assertion(TransactionBillDeviceRechargeStatus.BLUETOOTH.getStatus().equals(bill.getDeviceRechargeStatus()), "设备已选择蓝牙充值，请使用蓝牙进行充值");

            Boolean result = transactionTemplate.execute(status -> {
                // 更新套餐生效时间
                DeviceVO device = deviceService.selectSmDeviceByDeviceId(bill.getDeviceId());
                this.updateSuitTimeBeforeDevice(billId, device.getExpireTime());

                // 修改设备充值状态：成功
                int updateRecharge = transactionBillMapper.updateDeviceRechargeStatus(bill.getBillId(), TransactionBillDeviceRechargeStatus.SUCCESS.getStatus());
                ServiceUtil.assertion(updateRecharge != 1, "订单状态发生变化，请稍后重试");

                try {
                    // 设备充值
                    return deviceService.addTime(bill.getDeviceId(), bill.toSecondSuitTime(), true);
                } catch (Exception e) {
                    // 失败，回滚
                    status.setRollbackOnly();
                    return false;
                }
            });

            // 是否成功
            boolean success = result != null && result;

            if (success) {
                // 记录时长变化
                this.recordTime(bill, "用户充值");
            } else {
                // 修改设备充值状态为失败
                transactionBillMapper.updateDeviceRechargeStatus(billId, TransactionBillDeviceRechargeStatus.FAIL.getStatus());
            }

            return success;
        } finally {
            redisLock.unlock(RedisLockKey.RECHARGE_DEVICE, billId);
        }
    }

    /**
     * 在设备充值成功前，更新套餐时间
     * 开始时间 = 设备过期时间 - 套餐时间，过期时间 = 设备过期时间
     */
    private int updateSuitTimeBeforeDevice(Long billId, LocalDateTime deviceExpireTime) {
        TransactionBillVO bill = this.selectSmTransactionBillByBillId(billId);
        if (bill == null) {
            return 0;
        }
        // 计算开始使用的时间
        LocalDateTime now = LocalDateTime.now();
        LocalDateTime startTime = deviceExpireTime == null || now.isAfter(deviceExpireTime) ? now : deviceExpireTime;

        LocalDateTime endTime = startTime.plusSeconds(bill.toSecondSuitTime());
        TransactionBill data = new TransactionBill();
        data.setBillId(billId);
        data.setSuitStartTime(startTime);
        data.setSuitEndTime(endTime);
        data.setSuitExpireTime(endTime);
        return transactionBillMapper.updateSmTransactionBill(data);
    }

    /**
     * 获取支付结果
     * @param billNo 订单编号
     * @return 支付结果
     */
    @Override
    public boolean getPayResult(String billNo) {
        // 查询订单
        TransactionBill bill = transactionBillMapper.selectSmTransactionBillByBillNo(billNo);
        ServiceUtil.assertion(bill == null, "订单不存在");

        if (TransactionBillPayType.WECHAT.getType().equals(bill.getChannelId())) {
            // 微信支付
            Transaction transaction = wxPayService.queryOrderByOutTradeNo(billNo);
            return wxPayService.isSuccess(transaction);
        }
        // ... TODO 其他支付方式

        return false;
    }

    /**
     * 充值成功
     * @param billNo 充值订单编号
     */
    @Override
    public void rechargeSuccess(String billNo, Date payTime) {
        log.info("充值成功,billNo:{}", billNo);
        TransactionBill bill = transactionBillMapper.selectSmTransactionBillByBillNo(billNo);
        ServiceUtil.assertion(bill == null || !TransactionBillType.RECHARGE.getType().equals(bill.getType()), "充值订单不存在");

        Integer result = transactionTemplate.execute(status -> {
            // 修改订单状态
            int updateCount = transactionBillMapper.rechargeSuccess(bill.getBillId(), payTime);
            ServiceUtil.assertion(updateCount != 1, "修改订单状态失败，请刷新后重试");

            // 商户余额增加
            userService.addBalance(bill.getMchId(), bill.getArrivalAmount(), String.format("订单充值:%s", billNo));

            // 记录下充值后的余额
            SmUserVo user = userService.selectSmUserByUserId(bill.getMchId());
            updateAfterBalance(bill.getBillId(), user.getBalance());

            return updateCount;
        });

        if (result != null && result == 1) {
            // 设备充值
            rechargeDevice(bill.getBillId());
        }
    }

    /**
     * 记录下操作后的余额
     * @param billId 订单id
     * @param balance 余额
     */
    private void updateAfterBalance(Long billId, BigDecimal balance) {
        TransactionBill data = new TransactionBill();
        data.setBillId(billId);
        data.setAfterBalance(balance);
        transactionBillMapper.updateSmTransactionBill(data);
    }

    /**
     * 取消充值订单
     * @param billNo 订单编号
     * @param status 取消后的状态
     */
    @Override
    @Transactional
    public int cancelRecharge(String billNo, TransactionBillStatus status) {
        ServiceUtil.assertion(!TransactionBillStatus.isCancel(status.getStatus()), "参数错误：状态并非取消状态");

        TransactionBill bill = transactionBillMapper.selectSmTransactionBillByBillNo(billNo);
        ServiceUtil.assertion(bill == null, "订单不存在");
        // 用户自行取消，验证身份
        ServiceUtil.assertion(TransactionBillStatus.CANCELED.equals(status) && !UserUtil.hasFrontUser(bill.getUserId()), "这不是您的订单");

        List<String> allowCancel = Arrays.asList(TransactionBillStatus.UNPAID.getStatus(), TransactionBillStatus.PAYING.getStatus());
        ServiceUtil.assertion(!allowCancel.contains(bill.getStatus()), "订单状态非未支付或支付中，不允许取消");
        ServiceUtil.assertion(!TransactionBillType.RECHARGE.getType().equals(bill.getType()), "非充值订单，不允许取消");

        // 执行取消订单
        int updateCount = transactionBillMapper.cancelRecharge(billNo, status.getStatus());
        ServiceUtil.assertion(updateCount != 1, "订单取消失败，请刷新后重试");

        // 关闭支付订单
        if (TransactionBillStatus.PAYING.getStatus().equals(bill.getStatus())) {
            this.closeOrder(billNo);
        }

        return updateCount;
    }

    @Override
    public TransactionBillVO selectSmTransactionBillByBillNo(String billNo) {
        return transactionBillMapper.selectSmTransactionBillByBillNo(billNo);
    }

    @Override
    public List<TransactionBillVO> selectNoPayedRecharge() {
        TransactionBillQuery dto = new TransactionBillQuery();
        dto.setStatusList(Arrays.asList(TransactionBillStatus.UNPAID.getStatus(), TransactionBillStatus.PAYING.getStatus()));
        dto.setType(TransactionBillType.RECHARGE.getType());
        return this.selectSmTransactionBillList(dto);
    }

    /**
     * 查询设备充值失败列表
     *
     * @param userId
     * @return
     */
    @Override
    public List<TransactionBillVO> selectDeviceRechargeFailList(Long userId) {
        if (userId == null) {
            return Collections.emptyList();
        }
        // 查询充值失败列表
        TransactionBillQuery dto = new TransactionBillQuery();
        dto.setUserId(userId);
        dto.setStatus(TransactionBillStatus.SUCCESS.getStatus());
        dto.setDeviceRechargeStatus(TransactionBillDeviceRechargeStatus.FAIL.getStatus());
        dto.setType(TransactionBillType.RECHARGE.getType());
        List<TransactionBillVO> list = this.selectSmTransactionBillList(dto);

        if (!CollectionUtils.isEmptyElement(list)) {
            // 修改状态为蓝牙充值
            transactionBillMapper.updateDeviceRechargeStatusByIds(
                    list.stream().map(TransactionBillVO::getBillId).collect(Collectors.toList()),
                    TransactionBillDeviceRechargeStatus.BLUETOOTH.getStatus());
        }

        // 查询蓝牙充值列表
        dto.setDeviceRechargeStatus(TransactionBillDeviceRechargeStatus.BLUETOOTH.getStatus());
        return selectSmTransactionBillList(dto);
    }


    /**
     * 查询支付结果，直到过期
     *
     * @param billNo     订单编号
     * @param delay      延迟时间
     * @param timeUnit   时间单位
     */
    @Override
    public void refreshPayResultBeforeExpire(String billNo, int delay, TimeUnit timeUnit) {
        TransactionBill bill = transactionBillMapper.selectSmTransactionBillByBillNo(billNo);
        if (bill == null) {
            log.error("获取订单{}为空", billNo);
            return;
        }
        Date now = new Date();
        boolean payResult = false;

        // 判断是否支付成功
        try {
            if (TransactionBillStatus.PAYING.getStatus().equals(bill.getStatus())) {
                payResult = getPayResult(bill.getBillNo());
                if (payResult) {
                    // 支付成功
                    this.rechargeSuccess(bill.getBillNo(), now);
                    return;
                }
            }
        } catch (Exception e) {
            log.error("刷新支付结果出错：{}", e.getMessage());
        }

        // 未支付成功
        if (!payResult) {
            if (bill.getExpireTime() != null && now.compareTo(bill.getExpireTime()) < 0) {
                // 未过期，继续延迟查询
                log.info("{}支付失败，未过期，继续延迟查询", billNo);
                scheduledExecutorService.schedule(() -> {
                    this.refreshPayResultBeforeExpire(billNo, delay , timeUnit);
                }, delay, timeUnit);
            } else {
                log.info("{}支付失败，过期取消订单", billNo);
                // 过期取消订单
                int updateCount = transactionBillMapper.cancelRecharge(billNo, TransactionBillStatus.SYS_CANCELED.getStatus());
                log.info("过期取消订单，结果：{}", updateCount);
                if (updateCount == 1) {
                    // 关闭支付订单
                    this.closeOrder(billNo);
                }
            }
        }
    }

    /**
     * 查询数量并映射
     *
     * @param query     查询条件
     * @param keyMapper 映射方法
     */
    @Override
    public <K> Map<K, BillCountVo> selectCountMap(TransactionBillQuery query, Function<? super BillCountVo, ? extends K> keyMapper) {
        List<BillCountVo> list = this.selectCount(query);
        if (CollectionUtils.isEmptyElement(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(keyMapper, item -> item));
    }

    // 关闭订单
    private void closeOrder(String billNo) {
        TransactionBill bill = transactionBillMapper.selectSmTransactionBillByBillNo(billNo);
        if (bill == null) {
            return;
        }

        if (TransactionBillPayType.WECHAT.getType().equals(bill.getChannelId())) {
            wxPayService.closeOrder(billNo);
        }
    }


    /**
     * 蓝牙充值成功
     *
     * @param billNo
     * @return
     */
    @Override
    public boolean bluetoothRechargeSuccess(String billNo) {
        TransactionBillVO bill = selectSmTransactionBillByBillNo(billNo);
        ServiceUtil.assertion(bill == null, "订单不存在", 10001);
        ServiceUtil.assertion(!TransactionBillType.RECHARGE.getType().equals(bill.getType()), "该订单不是充值订单", 10002);
        ServiceUtil.assertion(TransactionBillDeviceRechargeStatus.SUCCESS.getStatus().equals(bill.getDeviceRechargeStatus()), "设备已充值成功，请勿重复充值", 10003);
        ServiceUtil.assertion(!TransactionBillStatus.SUCCESS.getStatus().equals(bill.getStatus()), "订单尚未充值成功", 10004);

        Boolean execute = transactionTemplate.execute(status -> {
            // 更新设备充值状态
            boolean result = transactionBillMapper.bluetoothRechargeSuccess(billNo) == 1;
            ServiceUtil.assertion(!result, "蓝牙充值回调失败");

            // 更新套餐时间
            DeviceVO afterDevice = deviceService.selectSmDeviceByDeviceId(bill.getDeviceId());
            this.updateSuitTimeBeforeDevice(bill.getBillId(), afterDevice.getExpireTime());

            // 修改剩余时间失败
            boolean addTime = deviceService.addTime(bill.getDeviceId(), bill.toSecondSuitTime(), false);
            ServiceUtil.assertion(!addTime, "修改剩余时间失败");

            return result;
        });

        // 是否成功
        boolean success = execute != null && execute;
        // 成功后操作
        if (success) {
            // 时长变化记录
            this.recordTime(bill, "蓝牙手动充值");
        }

        return success;
    }

    // 记录时长变化记录
    private void recordTime(TransactionBillVO bill, String reason) {
        scheduledExecutorService.schedule(() -> {
            DeviceVO device = deviceService.selectSmDeviceByDeviceId(bill.getDeviceId());
            if (device == null) {
                return;
            }
            SmUserVo user = userService.selectSmUserByUserId(bill.getUserId());
            recordTimeService.insertRecordTime(recordTimeConverter.toRecordTime(device, bill.toSecondSuitTime(), reason, user));
        }, 1L, TimeUnit.SECONDS);
    }

    @Override
    public int refund(BillRefundDTO dto) {
        // 校验订单
        TransactionBillVO bill = this.selectSmTransactionBillByBillId(dto.getBillId());
        if (bill == null) {
            return 0;
        }
        ServiceUtil.assertion(dto.getRefundAmount().compareTo(bill.getMoney()) > 0, "退款金额不允许大于订单金额");
        ServiceUtil.assertion(!TransactionBillStatus.SUCCESS.getStatus().equals(bill.getStatus()), "当前订单状态不允许退款");

        Integer result = transactionTemplate.execute(status -> {
            // 修改订单状态
            TransactionBill data = new TransactionBill();
            data.setStatus(TransactionBillStatus.REFUNDING.getStatus());
            TransactionBillQuery billQuery = new TransactionBillQuery();
            billQuery.setBillId(dto.getBillId());
            billQuery.setStatus(TransactionBillStatus.SUCCESS.getStatus());
            int updateBill = this.updateByQuery(data, billQuery);
            ServiceUtil.assertion(updateBill != 1, "修改订单状态失败");

            // 创建退款订单
            Refund refund = refundConverter.toPo(dto);
            int updateRefund = refundService.insertRefund(refund);
            ServiceUtil.assertion(updateRefund != 1, "创建退款订单失败");
            RefundVO refundVO = refundService.selectRefundByRefundNo(refund.getRefundNo());

            // 商户余额按照比例扣减
            userService.subtractBalance(bill.getMchId(), refund.getMchAmount(), "退款");

            // 修改原订单的退款金额和退款手续费
            int updateRefundAmount = this.addRefundAmount(bill.getBillId(), refund.getAmount(), refund.getMchAmount(), refund.getServiceAmount());
            ServiceUtil.assertion(updateRefundAmount != 1, "修改原订单的退款金额和退款手续费失败");

            // 发起退款
            if (TransactionBillPayType.WECHAT.getType().equals(bill.getChannelId())) {
                wxPayService.refund(refundVO);
            } else {
                throw new ServiceException("当前支付方式不支持退款");
            }
            return 1;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int updateByQuery(TransactionBill data, TransactionBillQuery query) {
        if (query == null) {
            return 0;
        }
        return transactionBillMapper.updateByQuery(data, query);
    }

    @Override
    @Transactional
    public int refreshPayResult(String billNo) {
        return this.refreshPayResult(this.selectSmTransactionBillByBillNo(billNo));
    }

    @Override
    public int refreshPayResult(List<TransactionBillVO> billList) {
        if (CollectionUtils.isEmptyElement(billList)) {
            return 0;
        }

        // 获取支付结果，并判断是否支付成功，若成功则更新数据
        Date now = new Date();
        for (TransactionBillVO bill : billList) {
            if (TransactionBillStatus.PAYING.getStatus().equals(bill.getStatus()) || TransactionBillStatus.UNPAID.getStatus().equals(bill.getStatus())  ) {
                boolean payResult = getPayResult(bill.getBillNo());
                if (payResult) {
                    this.rechargeSuccess(bill.getBillNo(), now);
                }
            }
        }
        return 1;
    }

    @Override
    public int addRefundAmount(Long billId, BigDecimal refundAmount, BigDecimal refundMchAmount, BigDecimal refundServiceAmount) {
        if (billId == null) {
            return 0;
        }
        return transactionBillMapper.addRefundAmount(billId, refundAmount, refundMchAmount, refundServiceAmount);
    }

    @Override
    public List<TransactionBillVO> selectUsingBill(Long userId) {
        TransactionBillQuery query = new TransactionBillQuery();
        query.setUserId(userId);
        return this.selectUsingBill(query);
    }

    @Override
    public List<TransactionBillVO> selectUsingBill(TransactionBillQuery query) {
        query.setUsing(true);
        query.setStatus(TransactionBillStatus.SUCCESS.getStatus());
        return this.selectSmTransactionBillList(query);
    }

    @Override
    public int batchEndBillByDevice(Long deviceId) {
        if (deviceId == null) {
            return 0;
        }
        LocalDateTime now = LocalDateTime.now();
        TransactionBillQuery query = new TransactionBillQuery();
        query.setDeviceId(deviceId);
        query.setEndSuitEndTime(now);   // 套餐结束时间在这之后的，都会被关闭
        List<TransactionBillVO> billList = this.selectSmTransactionBillList(query);
        if (CollectionUtils.isNotEmptyElement(billList)) {
            return transactionBillMapper.batchUpdateSuitEndTime(billList.stream().map(TransactionBill::getBillId).collect(Collectors.toList()), now);
        }
        return 0;
    }

    @Override
    public UserWithdrawServiceVO getUserWithdrawService(Long userId, Long channelId) {
        SmUserVo user = userService.selectSmUserByUserId(userId);
        ChannelVO channel = channelService.selectSmChannelByChannelId(channelId);
        return this.getUserWithdrawService(user, channel);
    }

    @Override
    public PrepayWithRequestPaymentResponse wxPay(TransactionBill bill) {
        ServiceUtil.assertion(bill == null || bill.getBillId() == null, "订单不存在");
        Long lockKey = bill.getBillId();
        ServiceUtil.assertion(!redisLock.lock(RedisLockKey.PREPAY, lockKey), "当前订单正在支付，请稍后重试");
        try {
            ServiceUtil.assertion(!Objects.equals(bill.getChannelId(), TransactionBillPayType.WECHAT.getType()), "该订单不是微信支付订单");
            ServiceUtil.assertion(!TransactionBillStatus.UNPAID.getStatus().equals(bill.getStatus()), "只能支付未支付的订单");
            ServiceUtil.assertion(!TransactionBillType.RECHARGE.getType().equals(bill.getType()), "只能支付充值订单");


            PrepayWithRequestPaymentResponse result = transactionTemplate.execute(status -> {
                // 修改状态为支付中
                int paying = this.paying(bill.getBillId());
                ServiceUtil.assertion(paying != 1, "更新支付状态失败");

                // 发起支付
                return wxPayService.prepayWithRequestPayment(bill);
            });

            // 每隔20秒查询支付结果，直到过期，首次10秒查询
            if (result != null) {
                scheduledExecutorService.schedule(() -> {
                    this.refreshPayResultBeforeExpire(bill.getBillNo(), 20, TimeUnit.SECONDS);
                }, 10, TimeUnit.SECONDS);
            }

            return result;
        } finally {
            redisLock.unlock(RedisLockKey.PREPAY, lockKey);
        }
    }
}
